package tsp.csc4509.tpnote;

import java.io.Serializable;
import java.util.Objects;

import tsp.csc4509.blockfile.Block;
import tsp.csc4509.blockfile.BlockFile;

/**
 * Classe qui décrit la requête envoyée par un client à un serveur pour demander
 * l'envoi d'un bloc du fichier.
 * @author Eric Lallet.
 *
 */
public class Request implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * nom du fichier contenant le bloc à envoyer.
	 */
	private String filename;
	/**
	 * offset (en octets) des données à envoyer dans le fichier.
	 */
	private long offset;
	/**
	 * tailles (en octets) des données à envoyer.
	 */
	private int size;
	
	/**
	 * Contructeur pour initialiser le le nom du fichier, l'offset et la taille.
	 * @param blockFile
	 *             BlockFile du fichier à importer.
	 * @param block
	 *             Block à importer.
	 */
	public Request(final BlockFile blockFile, final Block block) {
		super();
		this.filename = blockFile.getFileName();
		this.offset = block.getOffset();
		this.size = block.getSize();
	}
	
	/**
	 * Contructeur pour initialiser le le nom du fichier, l'offset et la taille.
	 * Utiliser pour tester sans avoir à constuire de Block.
	 * @param filename
	 *           nom du fichier à envoyer.
	 * @param offset
	 *           offset (en octets) des données à envoyer.
	 * @param size
	 *           tailles (en octets) des données à envoyer.
	 */
	public Request(final String filename, final long offset, final int size) {
		super();
		this.filename = filename;
		this.offset = offset;
		this.size = size;
	}
	
	/**
	 * getter sur le nom du fichier.
	 * @return
	 *       le nom du fichier contenant le bloc à envoyer.
	 */
	public String getFilename() {
		return filename;
	}
	
	/**
	 * getter sur l'offset des données à envoyer.
	 * @return
	 *        offset (en octets) des données à envoyer.
	 */
	public long getOffset() {
		return offset;
	}
	
	/**
	 * getter sur la taille des données à envoyer.
	 * @return
	 *       tailles (en octets) des données à envoyer.
	 */
	public int getSize() {
		return size;
	}

	@Override
	public String toString() {
		return "Request [filename=" + filename + ", offset=" + offset + ", size=" + size + "]";
	}

	@Override
	public int hashCode() {
		return Objects.hash(filename, offset, size);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Request other = (Request) obj;
		return Objects.equals(filename, other.filename) && offset == other.offset && size == other.size;
	}
	
	

}
