package tsp.csc4509.tpnote;


import static tsp.csc4509.common.Log.GEN;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

public class Server {
//public class Server extends tsp.csc4509.tpnote.aide.ServerAide {

	/**
	 * Nombre de threads lancés dans le pool.
	 */
	private static final int NBTHREADS = 4;

	/**
	 * le canal gérant les accepts du serveur.
	 */
	private ServerSocketChannel listenChannel;

	/**
	 * Constructeur de Server qui ouvre le canal du serveur
	 * et le lie au port passé en paramètre.
	 * @param port
	 *           port que doit occuper le serveur.
	 * @throws IOException
	 *           toutes les exceptions d'entrées/sorties qui peuvent se produire.
	 */
	public Server(final int port) throws IOException {
		listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
	}

	/**
	 * Accepte la connexion d'un client sur le canal listenChannel.
	 * @return
	 *        le canal relié au client qui vient de se connecter.
	 * @throws IOException
	 *        toutes les exceptions d'entrées/sorties qui peuvent se produire.
	 */
	public SocketChannel accept() throws IOException {
		GEN.info("Server::accept()");

		return listenChannel.accept();
	}

	/**
	 * Implémente le service:
	 *   1. Crée la file d'attente pour envoyer les demandes aux threads;
	 *   2. Lance un pool de NBTHREADS threads;
	 *   3. Accepte tous les clients qui se connectent et place leur canaux
	 *      dans la file d'attente.
	 * @throws IOException
	 *         toutes les exceptions d'entrées/sorties qui peuvent se produire.
	 * @throws InterruptedException
	 *         exception due la gestion de la BlockingQueue.
	 */
	public void service() throws IOException, InterruptedException { // le service n'est pas robuste: il ne récupère pas des exceptions
		GEN.info("Server::service() début");
		BlockingQueue<SocketChannel> queue = new LinkedBlockingQueue<SocketChannel>();
		for (int nb = 0; nb < NBTHREADS; nb++) {
			new Thread(new ServerRunnable(queue)).start();
		}
		GEN.info("Server::service() threads lancés");

		while (true) {
			SocketChannel clientChan = accept();
			GEN.info("Server::service() ajout d'un client en file d'attente");
			queue.put(clientChan);
		}
	}


	// ***************** getters sur les attributs ajoutés à la classe pour l'écriture des tests
	// ils ne doivent pas être utilisés dans votre code. Seule la classe Serveur doit manipuler ces attributs. 
	/**
	 * Getter sur listenChannel.
	 * @return
	 *       l'attribut listenChannel.
	 */
	public ServerSocketChannel getListenChannel() {
		return listenChannel;
	}


}
