//CHECKSTYLE:OFF
package tsp.csc4509.tpnote;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static tsp.csc4509.common.Log.TEST;
import static tsp.csc4509.common.Log.GEN;
import static tsp.csc4509.common.Log.COMM;

import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.apache.logging.log4j.Level;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import tsp.csc4509.blockfile.BlockFile;
import tsp.csc4509.common.Log;
import tsp.csc4509.divers.AppliGenerateur;
import tsp.csc4509.tcp.TcpServerSynchrone;
import tsp.csc4509.tcp.TcpSocketSynchrone;


public class TestServer {
	static final public String TESTFILE = "fichier40k";
	static final public String EMPYFILE="fichierVide";
	static final public int VALUE = 10000;
	static final public int OFFSET = 20000;
	static final public int SIZE = 10000;

	@BeforeClass
	static public void before() throws IOException {
		Log.setLevel(TEST, Level.INFO);
		Log.setLevel(GEN, Level.INFO);
		Log.setLevel(COMM, Level.INFO);

		// mise en place du fichier de test
		
		String fileName = TESTFILE;
		AppliGenerateur.generateFile(BlockFile.SOURCE + "/" + fileName, VALUE);
		BlockFile.createFile(fileName);
		
		fileName  = EMPYFILE;
		AppliGenerateur.generateFile(BlockFile.SOURCE + "/" + fileName, 0);
		BlockFile.createFile(fileName);
	}

	/**
	 * Test unitaire du constructeur.
	 * @throws InterruptedException 
	 * @throws IOException 
	 */
//	@Ignore
	@Test
	public void testServer() throws InterruptedException, IOException  {
		TEST.info("[TestServer::testServer] début");
		Runnable serveurRunnable = new Runnable() {
			@Override
			public void run() {
				TEST.info("[TestServer::testServer] serveur en route");
				try {
					Server server = new Server(ServerInfo.SERVER1.getPort());
					ServerSocketChannel listenChannel = server.getListenChannel();
					assertTrue("listenChannel n'est pas null", listenChannel != null);
					
					SocketChannel chan = listenChannel.accept();
					assertTrue("un client a été accépté", chan != null);
					
				} catch (Exception e1) {
					return;
				}
				
			}

		};
		
		Thread serveur = new Thread(serveurRunnable);
		serveur.start();
		Thread.sleep(1000);
		
		TcpSocketSynchrone client = new TcpSocketSynchrone(ServerInfo.SERVER1.getHostname(), ServerInfo.SERVER1.getPort());
		client.close();
		
		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}
	}

	@Test
	public void testAccept() throws InterruptedException, IOException  {
		TEST.info("[TestServer::testAccept] début");
		Runnable serveurRunnable = new Runnable() {
			@Override
			public void run() {
				TEST.info("[TestServer::testServer] serveur en route");
				try {
					Server server = new Server(ServerInfo.SERVER1.getPort());
					ServerSocketChannel listenChannel = server.getListenChannel();
					assertTrue("listenChannel n'est pas null", listenChannel != null);
					
					SocketChannel chan = server.accept();
					assertTrue("un client a été accépté", chan != null);
					
				} catch (Exception e1) {
					return;
				}
				
			}

		};
		
		Thread serveur = new Thread(serveurRunnable);
		serveur.start();
		Thread.sleep(1000);
		
		TcpSocketSynchrone client = new TcpSocketSynchrone(ServerInfo.SERVER1.getHostname(), ServerInfo.SERVER1.getPort());
		client.close();
		
		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}
	}
	
	
	
}
